/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;

import org.apache.ws.Soap1_1Constants;
import org.apache.ws.resource.faults.FaultException;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.util.i18n.Messages;

/**
 * A base class that can be extended by portType implementations.
 *
 * @author Ian Springer
 */
public abstract class AbstractPortType
{
   /** DOCUMENT_ME */
   public static final Messages MSG               = MessagesImpl.getInstance(  );
   private ResourceContext      m_resourceContext;
   private Resource             m_resource;

   /**
    * Creates a new {@link AbstractPortType} object.
    *
    * @param resourceContext resource context
    */
   public AbstractPortType( ResourceContext resourceContext )
   {
      m_resourceContext = resourceContext;
      try
      {
         m_resource = m_resourceContext.getResource(  );
      }
      catch ( ResourceUnknownException rue )
      {
         throw new org.apache.ws.resource.properties.faults.ResourceUnknownFaultException( getNamespaceSet(  ),
                                                                                           rue );
      }
      catch ( ResourceException re )
      {
         throw new FaultException( Soap1_1Constants.FAULT_SERVER, "Internal server error." );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Resource getResource(  )
   {
      return m_resource;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceContext getResourceContext(  )
   {
      return m_resourceContext;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected abstract NamespaceVersionHolder getNamespaceSet(  );
}