/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;

import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.i18n.Messages;

/**
 * Indicates that a {@link Resource} instance with the specified id
 * was not found in the corresponding {@link ResourceHome}.
 */
public class ResourceUnknownException
   extends ResourceException
{
   /** DOCUMENT_ME */
   public static final Messages MSG           = MessagesImpl.getInstance(  );
   private Object               m_resourceId;
   private String               m_serviceName;

   /**
    * @param resourceId the resource id that was unknown
    * @param serviceName the name of the service associated with the home where we tried to lookup the resource
    */
   public ResourceUnknownException( Object resourceId,
                                    String serviceName )
   {
      super( MSG.getMessage( Keys.RESOURCE_NOT_FOUND, resourceId, serviceName ) );
      m_resourceId     = resourceId;
      m_serviceName    = serviceName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object getResourceId(  )
   {
      return m_resourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServiceName(  )
   {
      return m_serviceName;
   }
}