/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.lifetime.impl;

import org.apache.ws.resource.lifetime.ResourceTerminationEvent;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.xmlbeans.XmlObject;
import java.util.Calendar;

/**
 * @author Sal Campana
 */
public abstract class AbstractResourceTerminationEvent
   implements ResourceTerminationEvent
{
   private Calendar m_terminationTime;
   private Object   m_reason;
   private Object   m_resourceId;

   /**
    * Sets termination time and reason to passed in params.
    *
    * @param resourceId The resource Id of the resource being destroyed.
    * @param terminationTime
    * @param reason
    */
   public AbstractResourceTerminationEvent( Object   resourceId,
                                            Calendar terminationTime,
                                            Object   reason )
   {
      m_terminationTime    = terminationTime;
      m_reason             = reason;
      m_resourceId         = resourceId;
   }

   /**
    * Set the reson to the object specified and termination time is set upon construction.
    *
    * @param resourceId The resource Id of the resource being destroyed.
    * @param reason
    */
   public AbstractResourceTerminationEvent( Object resourceId,
                                            Object reason )
   {
      this( resourceId,
            Calendar.getInstance(  ), reason );
   }

   /**
    * Constructor creates termination time upon construction.  Reason is null.
    *
    * @param resourceId The resource Id of the resource being destroyed.
    */
   public AbstractResourceTerminationEvent( Object resourceId )
   {
      this( resourceId,
            Calendar.getInstance(  ), null );
   }

   /**
    * Returns the reason for temrination.
    * <p/>
    * minOccurs="0" maxOccurs="1"
    *
    * @return The reason for termination.
    */
   public Object getReason(  )
   {
      return m_reason;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object getResourceId(  )
   {
      return m_resourceId;
   }

   /**
    * Returns the termination time.
    * <p/>
    * minOccurs="1" maxOccurs="1" nillable="true"
    *
    * @return The termination time represented as a Calendar instance.
    */
   public Calendar getTerminationTime(  )
   {
      return m_terminationTime;
   }

   /**
    * DOCUMENT_ME
    *
    * @param namespaces DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected abstract XmlObject getTerminationNotifDocXmlBean( NamespaceVersionHolder namespaces );
}