/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.tool.velocity;

import org.apache.ws.resource.ResourceDefinition;
import org.apache.ws.util.XmlBeanNameUtils;
import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A bean that contains all properties that are needed by our Velocity templates.
 *
 * @author Sal Campana
 */
public class ServiceProperties
{
   /** DOCUMENT_ME */
   public static final String SPEC_NAMESPACE_MEMBER_NAME = "SPEC_NAMESPACE_SET";

   /** DOCUMENT_ME */
   String m_packageName;

   /** DOCUMENT_ME */
   String m_serviceName;

   /** DOCUMENT_ME */
   String m_serviceImplements;

   /** DOCUMENT_ME */
   String m_namespace;

   /** DOCUMENT_ME */
   String m_prefix;

   /** DOCUMENT_ME */
   Map m_customMappings;

   /** DOCUMENT_ME */
   Map m_propertyMap;

   /** DOCUMENT_ME */
   String m_resourceImplements;

   /** DOCUMENT_ME */
   String m_propertyDocumentName;

   /** DOCUMENT_ME */
   String m_wsdlName;

   /** DOCUMENT_ME */
   QName m_resourcekey;

   /** DOCUMENT_ME */
   List m_resourceIncludes = new ArrayList(  );

   /** DOCUMENT_ME */
   List m_serviceIncludes = new ArrayList(  );

   /** DOCUMENT_ME */
   List m_resourceInitIncludes = new ArrayList(  );

   /** DOCUMENT_ME */
   List                       m_abstractResourceInitIncludes    = new ArrayList(  );
   private boolean            m_hasProperties;
   private String             m_endpointServiceName;
   private String             m_endpointURL;
   private String             m_jarDir;
   private String             m_classesDir;
   private String             m_srcDir;
   private String             m_jarFileName;
   private ResourceDefinition m_resourceDef;
   private String             m_namespaceVersionHolder;
   private QName              m_serviceQName;
   private String             m_portName;
   private QName              m_portTypeQName;
   private List               m_implementedProps                = new ArrayList(  );
   private String             m_webbAppName;
   private String             m_namespaceVersionHolderClassName;
   private String             m_port;

   /**
    * Creates a new {@link ServiceProperties} object.
    *
    * @param resourceDef DOCUMENT_ME
    */
   public ServiceProperties( ResourceDefinition resourceDef )
   {
      m_resourceDef = resourceDef;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public List getAbstractResourceInitIncludeLocations(  )
   {
      return m_abstractResourceInitIncludes;
   }

   /**
    * DOCUMENT_ME
    *
    * @param classesDir DOCUMENT_ME
    */
   public void setClassesDir( String classesDir )
   {
      m_classesDir = classesDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getClassesDir(  )
   {
      return m_classesDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @param customMappings DOCUMENT_ME
    */
   public void setCustomMappings( Map customMappings )
   {
      this.m_customMappings = customMappings;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Map getCustomMappings(  )
   {
      return m_customMappings;
   }

   /**
    * DOCUMENT_ME
    *
    * @param prop_qname DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getDocumentElementXmlBeanClassName( QName prop_qname )
   {
      return XmlBeanNameUtils.getDocumentElementXmlBeanClassName( prop_qname );
   }

   /**
    * DOCUMENT_ME
    *
    * @param serviceName DOCUMENT_ME
    */
   public void setEndpointServiceName( String serviceName )
   {
      m_endpointServiceName = serviceName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getEndpointServiceName(  )
   {
      return m_endpointServiceName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param endpointURL DOCUMENT_ME
    */
   public void setEndpointURL( String endpointURL )
   {
      m_endpointURL = endpointURL;
      try
      {
         setPort( Integer.toString( new java.net.URL( endpointURL ).getPort(  ) ) );
      }
      catch ( java.net.MalformedURLException e )
      {
         throw new RuntimeException( e );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getEndpointURL(  )
   {
      return m_endpointURL;
   }

   /**
    * DOCUMENT_ME
    *
    * @param hasProps DOCUMENT_ME
    */
   public void setHasProperties( boolean hasProps )
   {
      m_hasProperties = hasProps;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean getHasProperties(  )
   {
      return m_hasProperties;
   }

   /**
    * Returns a Map of the properties which the implementor will
    * need to implement.
    *
    * @return The Map of props which need to be implenmented/
    */
   public Map getImplementableProperties(  )
   {
      Map implementableMap = new HashMap( m_propertyMap );
      for ( int i = 0; i < m_implementedProps.size(  ); i++ )
      {
         implementableMap.remove( ( (QName) m_implementedProps.get( i ) ).getLocalPart(  ).toUpperCase(  ) );
      }

      return implementableMap;
   }

   /**
    * Returns a list of only the properties which the implementor has defined
    * in their wsdl AND that ARE implemented for them.
    *
    * @return A list of the props which the implementor will not need to implement.
    */
   public List getImplementedProperties(  )
   {
      List list = new ArrayList(  );
      for ( int i = 0; i < m_implementedProps.size(  ); i++ )
      {
         if ( m_propertyMap.values(  ).contains( m_implementedProps.get( i ) ) )
         {
            list.add( m_implementedProps.get( i ) );
         }
      }

      return list;
   }

   /**
    * DOCUMENT_ME
    *
    * @param jarDir DOCUMENT_ME
    */
   public void setJarDir( String jarDir )
   {
      m_jarDir = jarDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getJarDir(  )
   {
      return m_jarDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @param jarFileName DOCUMENT_ME
    */
   public void setJarFileName( String jarFileName )
   {
      m_jarFileName = jarFileName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getJarFileName(  )
   {
      return m_jarFileName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param namespace DOCUMENT_ME
    */
   public void setNamespace( String namespace )
   {
      this.m_namespace = namespace;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getNamespace(  )
   {
      return m_namespace;
   }

   /**
    * DOCUMENT_ME
    *
    * @param namespaceValueHolder DOCUMENT_ME
    */
   public void setNamespaceVersionHolder( Class namespaceValueHolder )
   {
      if ( namespaceValueHolder != null )
      {
         setNamespaceVersionHolderClassName( namespaceValueHolder );
         String className = namespaceValueHolder.getName(  );
         m_namespaceVersionHolder =
            "public static final " + className + " " + SPEC_NAMESPACE_MEMBER_NAME + " = new " + className + "();";
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getNamespaceVersionHolder(  )
   {
      return m_namespaceVersionHolder;
   }

   /**
    * DOCUMENT_ME
    *
    * @param namespaceValueHolder DOCUMENT_ME
    */
   public void setNamespaceVersionHolderClassName( Class namespaceValueHolder )
   {
      m_namespaceVersionHolderClassName = namespaceValueHolder.getName(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getNamespaceVersionHolderClassName(  )
   {
      return m_namespaceVersionHolderClassName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param packageName DOCUMENT_ME
    */
   public void setPackageName( String packageName )
   {
      this.m_packageName = packageName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getPackageName(  )
   {
      return m_packageName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param port DOCUMENT_ME
    */
   public void setPort( String port )
   {
      m_port = port;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getPort(  )
   {
      return m_port;
   }

   /**
    * DOCUMENT_ME
    *
    * @param portName DOCUMENT_ME
    */
   public void setPortName( String portName )
   {
      m_portName = portName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getPortName(  )
   {
      return m_portName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param portTypeQName DOCUMENT_ME
    */
   public void setPortTypeQName( QName portTypeQName )
   {
      m_portTypeQName = portTypeQName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getPortTypeQName(  )
   {
      return m_portTypeQName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param prefix DOCUMENT_ME
    */
   public void setPrefix( String prefix )
   {
      this.m_prefix = prefix;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getPrefix(  )
   {
      return m_prefix;
   }

   /**
    * DOCUMENT_ME
    *
    * @param propertyDocumentName DOCUMENT_ME
    */
   public void setPropertyDocumentName( String propertyDocumentName )
   {
      this.m_propertyDocumentName = propertyDocumentName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getPropertyDocumentName(  )
   {
      return m_propertyDocumentName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param propertyMap DOCUMENT_ME
    */
   public void setPropertyMap( Map propertyMap )
   {
      this.m_propertyMap = propertyMap;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Map getPropertyMap(  )
   {
      return m_propertyMap;
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourceImplements DOCUMENT_ME
    */
   public void setResourceImplements( String resourceImplements )
   {
      this.m_resourceImplements = resourceImplements;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getResourceImplements(  )
   {
      return m_resourceImplements;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public List getResourceIncludeFiles(  )
   {
      return m_resourceIncludes;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public List getResourceInitIncludeLocations(  )
   {
      return m_resourceInitIncludes;
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourcekey DOCUMENT_ME
    */
   public void setResourcekey( QName resourcekey )
   {
      this.m_resourcekey = resourcekey;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getResourcekey(  )
   {
      return m_resourcekey;
   }

   /**
    * DOCUMENT_ME
    *
    * @param serviceImplements DOCUMENT_ME
    */
   public void setServiceImplements( String serviceImplements )
   {
      this.m_serviceImplements = serviceImplements;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServiceImplements(  )
   {
      return m_serviceImplements;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public List getServiceIncludeFiles(  )
   {
      return m_serviceIncludes;
   }

   /**
    * DOCUMENT_ME
    *
    * @param serviceName DOCUMENT_ME
    */
   public void setServiceName( String serviceName )
   {
      this.m_serviceName = serviceName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServiceName(  )
   {
      return m_serviceName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param serviceQname DOCUMENT_ME
    */
   public void setServiceQName( QName serviceQname )
   {
      m_serviceQName = serviceQname;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getServiceQName(  )
   {
      return m_serviceQName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param srcDir DOCUMENT_ME
    */
   public void setSrcDir( String srcDir )
   {
      m_srcDir = srcDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getSrcDir(  )
   {
      return m_srcDir;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void setWebappName( String name )
   {
      m_webbAppName = name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getWebappName(  )
   {
      return m_webbAppName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param wsdlName DOCUMENT_ME
    */
   public void setWsdlName( String wsdlName )
   {
      this.m_wsdlName = wsdlName;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getWsdlName(  )
   {
      return m_wsdlName;
   }

   /**
    * DOCUMENT_ME
    *
    * @param abstractResourceInitInclude DOCUMENT_ME
    */
   public void addAbstractResourceInitIncludeLocation( String abstractResourceInitInclude )
   {
      m_abstractResourceInitIncludes.add( abstractResourceInitInclude );
   }

   /**
    * Use this method to tell the ServiceProperties which ResourceProperties we implement for the user.
    *
    * @param propQnameTerminationTime
    */
   public void addImplementedProperty( QName propQnameTerminationTime )
   {
      m_implementedProps.add( propQnameTerminationTime );
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourceInclude DOCUMENT_ME
    */
   public void addResourceIncludeFile( String resourceInclude )
   {
      m_resourceIncludes.add( resourceInclude );
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourceInitInclude DOCUMENT_ME
    */
   public void addResourceInitIncludeLocation( String resourceInitInclude )
   {
      m_resourceInitIncludes.add( resourceInitInclude );
   }

   /**
    * DOCUMENT_ME
    *
    * @param serviceInclude DOCUMENT_ME
    */
   public void addServiceIncludeFile( String serviceInclude )
   {
      m_serviceIncludes.add( serviceInclude );
   }

   /**
    * DOCUMENT_ME
    *
    * @param capability DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean implementsResourceCapability( QName capability )
   {
      return m_resourceDef.implementsResourceCapability( capability );
   }
}