/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.webapp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.jndi.XmlBeanJndiUtils;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import java.io.InputStream;

/**
 * A servlet context listener for the WSRF webapp that provides the hook for
 * initializing the WSRF JNDI context. Note, to configure this listener, add
 * the following lines to web.xml:
 *
 *   &lt;listener&gt;
 *     &lt;listener-class&gt;org.apache.ws.resource.webapp.WsrfServletContextListener&lt;/listener-class&gt;
 *   &lt;/listener&gt;
 *
 * @author Ian Springer
 */
public class WsrfServletContextListener
   implements ServletContextListener
{
   private static final Log      LOG = LogFactory.getLog( WsrfServletContextListener.class );
   private static final Messages MSG = MessagesImpl.getInstance(  );

   /**
    * DOCUMENT_ME
    *
    * @param contextEvent DOCUMENT_ME
    */
   public void contextDestroyed( ServletContextEvent contextEvent )
   {
      // intentionally empty
   }

   /**
    * DOCUMENT_ME
    *
    * @param contextEvent DOCUMENT_ME
    */
   public void contextInitialized( ServletContextEvent contextEvent )
   {
      String contextName = contextEvent.getServletContext(  ).getServletContextName(  );
      LOG.info( "Initializing the " + contextName + " webapp..." );
      LOG.info( MSG.getMessage( Keys.POPULATING_JNDI, XmlBeanJndiUtils.JNDI_CONFIG_FILENAME ) );
      try
      {
         InputStream jndiConfigStream =
            Thread.currentThread(  ).getContextClassLoader(  ).getResourceAsStream( XmlBeanJndiUtils.JNDI_CONFIG_FILENAME );
         XmlBeanJndiUtils.initFromInputStream( jndiConfigStream );
      }
      catch ( Exception e )
      {
         throw new RuntimeException( "Fatal error! Failed to initialize WSRF JNDI context.", e );
      }
   }
}